#!/usr/bin/env python
#
# test out all the bits in the njsmodule
#
# $Id: test.py,v 1.2 2001/11/13 14:49:10 d Exp $

import njs
import sys

print "test version", "$Revision: 1.2 $"
print "NJS version", njs.version
print "NJSmodule version", njs.__version__

print "=== test interpreter construction"

print "Creating interpreter"
opt = njs.create_default_options()
#opt.verbose = 2
opt.stacktrace_on_error = 1
#opt.debug_info = 1
#opt.annotate_assembler = 1
i = njs.create_interp(opt)
print "Interpeter is", i
print

print "=== test getting some values"

print "System is", i.get_var("System")
print "Infinity is", i.get_var("Infinity")
print "true is", i.eval("true")
print "false is", i.eval("false")
print "null is", i.eval("null")
print "NaN is", i.eval("0/0")
print "undefined is", i.eval("Math.blah")
print

print "=== test global methods"

def foo(*args):
	print "  now in foo()."
	print "  args are", `args`
	print "  returning 7"
	return 7
print "creating global method bar() -> foo()"
i.create_global_method("bar", foo)
print

print "result of evaluating 'bar(1,2,3)' is"
print i.eval("bar(1,2,3)")
print
	
print "=== test eval"

expression = "3 + 8 * 2"
print "Expression is", expression
try:
	print "Evaluated result is", i.eval(expression)
except njs.Error, e:
	print "Exception during eval:", e
print

print "=== test byte code compiling"

program = 'System.stdout.writeln("foo");'
print "Program is:", program
bc = i.compile_data_to_byte_code(program)
print "Byte code is:", repr(bc)
try:
	i.execute_byte_code(bc)
	print "Result is:", repr(i.result)
except njs.Error, e:
	print "Exception during byte code execution:", `e`
print

print "=== test defining a variable"

print "defining a variable"
i.set_var("Three", 3)
i.eval('System.stdout.writeln("   Three = " + Three);')
print

print "=== test classes and instances"

def constructorfunc(cls, interp, args):
	print "  constructorfunc: cls", cls, "interp", interp, "args", `args`
	return "context:"+`args`

cls = njs.create_class("the class context", constructorfunc)

def methods(cls, interp, ctxt, name, args):
	print "  methodfunc: cls", cls, "interp", interp, \
			"ctxt", `ctxt`, "name", `name`, "args", `args`
	if name == "retnative":
		print " returning complex python object"
		return {'foo':4}
	if name == "reterror":
		raise RuntimeError, "synthetic test error"
	if name == "nothing":
		return
	if name == 'm':
		return ("Yay","for","m")
	raise NameError, name

cls.define_generic_method(njs.CF_STATIC, methods)

print "constructing a class, and invoking a method"
i.define_class(cls, "c")
print i.eval('(new c("arg1")).m("arg2")')

print "constructing an instance 'manually' and invoke a method on it"
inst = cls.instantiate_class(i, "mycontext")

print "instance:", inst
i.set_var("inst", inst);
print "invoking inst.nothing()"
i.eval('inst.nothing("hello")')

print "inst's context is", `i.isa(inst, cls)`

print
print "=== test returning a python dictionary object through a python callback"

print "testing return of a dictionary from a callback function"
res = i.eval('c.retnative()')
print "got returned value:", `res`
print

print "=== test exception handling"

print 'exception tests:'
try:
	print "evaluating 'syntax error'"
	i.eval("syntax error")
except njs.Error, e:
	print "Exception during eval:", `e`, e
print

try:
	i.eval('c.reterror()')
except njs.Error, e:
	print "Exception during eval:", `e`, e
print

print "=== success"

